"Created by Hasan Besirovic"
"Functions to extract data and do frequency analysis"

from nptdms import TdmsFile
import numpy as np
import scipy.signal as sps

"""""
Function that extracts values of the desired channel and group of data from a Tdms file. 
"""""
def data_you_want(Tdms_file, name_of_dataGroup, name_of_dataChannel):
    tdms_file = TdmsFile(Tdms_file)
    for group in tdms_file.groups():
        if group == name_of_dataGroup:
            for channel in tdms_file.group_channels(group):
                C_Name = channel.properties["NI_ChannelName"]
                if C_Name == name_of_dataChannel:
                    data = channel.data
                    time = channel.time_track()
                    dt = channel.properties["wf_increment"]
                    break
            break
    return data, dt, time

"Welch on the data that was collected"
"""Difference between this and the next function is that this one take some specific datagroups and find and plots the 
power spectrum while the other takes all the groups and finds the peaks."""
def Welch_func(Tdms_file, Datagroups, overlap_None, nperseg_N, Min_plot_freq, Max_plot_freq,Natural_1_or_not_0):
    k = 0
    Pspektrum = []
    f = []
    Pspektrum_max = []
    f_max = []
    Pspektrum_secondMax = []
    f_secondMax = []
    Velocity = []
    Windowtype = []

    for file in Tdms_file:
        for i in range(len(Datagroups) / len(Tdms_file)):
            data = data_you_want(file, Datagroups[i + k], "Strain")

            if nperseg_N == "N":
                nperseg_N_lokke = data[0].size
            else:
                nperseg_N_lokke = data[0].size/nperseg_N

            dt = data[1]

            if Datagroups[i+k] == "8ms":
                f_lokke, Pxx = sps.welch(data[0], 1 / dt, window=('kaiser', 2), nperseg=nperseg_N_lokke,
                                         noverlap=overlap_None, scaling="spectrum")
                Windowtype.append('Kaiser')
            else:
                f_lokke, Pxx = sps.welch(data[0], 1 / dt, window='hann', nperseg=nperseg_N_lokke,
                                         noverlap=overlap_None,scaling="spectrum")
                Windowtype.append('Hanning')

            "Limit the frequency spectrum and add them for plotting"
            Pspektrum_lokke = np.sqrt(Pxx)[maxmin_freq(f_lokke, Min_plot_freq):maxmin_freq(f_lokke, Max_plot_freq)]
            f_lokke = f_lokke[maxmin_freq(f_lokke, Min_plot_freq):maxmin_freq(f_lokke, Max_plot_freq)]
            f.append(f_lokke)
            Pspektrum.append(Pspektrum_lokke)

            "Finding the spectrum peak with corresponding frequency"
            Pspektrum_max.append(max(Pspektrum_lokke))
            f_max.append(f_lokke[np.where(Pspektrum_lokke==max(Pspektrum_lokke))])

            "Finding the second max for the Natural Frequency sweep and flow velocity for the power"
            "spectrum plot from the calibrated constants"
            if Natural_1_or_not_0 == 1:
                peaks,_ = sps.find_peaks(Pspektrum_lokke,distance=len(Pspektrum_lokke)/4)
                if len(peaks) > 1:
                    Pspektrum_secondMax.append(sorted(Pspektrum_lokke[peaks])[-2])
                    f_secondMax.append(f_lokke[np.where(Pspektrum_lokke==sorted(Pspektrum_lokke[peaks])[-2])])
            else:
                Flow = 0.083528531 * round(np.mean(data_you_want(file,Datagroups[i+k],"Flow")[0]),5) - 0.165376967
                Velocity.append(Flow / 0.15 ** 2)

        k = k + len(Datagroups) / len(Tdms_file)
    return Pspektrum, f, Pspektrum_max, f_max, Pspektrum_secondMax, f_secondMax, Velocity, Windowtype

"""""
Function for taking all the groups from a Tdms file and calculating the FFT of a given channel. 
Then finding the peak of this frequency analysis. It also extracts the flow speed in the cross section at the foil.
First peak will move while second will be still at almost a constant place corresponding to the eigenfrequency.
"""""
def Welch_and_Max_of_data(Tdms_file, name_of_dataChannel,max_freq,min_freq,SamplesPerWindow):
    MAX_Amplitude = []
    Velocities_Measured = []
    Second_Peak_frequency = []
    First_Peak_frequency = []
    Velocities_Measured_First_Peak = []
    Errorbars_SecPeak = []
    Errorbars_FirstPeak = []

    for file in Tdms_file:
        tdms_file = TdmsFile(file)
        for group in tdms_file.groups():

            "Skipping certain files"
            if file == "" and group != "9_1ms":
                continue
            elif group == "10_6ms" or group == "4ms" or group == "15ms" or group == "16ms":
                continue

            for channel in tdms_file.group_channels(group):
                C_Name = channel.properties["NI_ChannelName"]
                if C_Name == name_of_dataChannel:
                    "Finding the second peak"
                    N = len(channel.data)
                    dt = channel.properties["wf_increment"]

                    "Choosing which groups to have another type of window on"
                    if group == "8ms":
                        f_lokke, Pspektrum_lokke = sps.welch(channel.data, 1 / dt,window=('kaiser',2),
                                                             nperseg=N,noverlap=None,scaling="spectrum")
                    else:
                        f_lokke, Pspektrum_lokke = sps.welch(channel.data, 1 / dt,window='hann',
                                                             nperseg=N,noverlap=None,scaling="spectrum")

                    "Finding the limits and limiting data"
                    Max_freq = maxmin_freq(f_lokke,max_freq)
                    Min_freq = maxmin_freq(f_lokke,min_freq)
                    Pspektrum = np.sqrt(Pspektrum_lokke)[Min_freq:Max_freq]
                    f = f_lokke[Min_freq:Max_freq]

                    "Finding the second peak"
                    Max_peak = max(Pspektrum)
                    MAX_Amplitude.append(Max_peak)
                    Second_Peak_frequency.append(f[np.where(Pspektrum == Max_peak)])

                    "Finding the first peak, Using a larger window to find the peak"
                    if group == "8ms":
                        f_hanning, Pspektrum_hanning = sps.welch(channel.data, 1 / dt,window=('kaiser',3),
                                                        nperseg=N/SamplesPerWindow, noverlap=None,scaling="spectrum")
                    else:
                        f_hanning, Pspektrum_hanning = sps.welch(channel.data, 1 / dt,window=('hann'),
                                                        nperseg=N/SamplesPerWindow, noverlap=None,scaling="spectrum")

                    Max_freq_H = maxmin_freq(f_hanning, max_freq)
                    Min_freq_H = maxmin_freq(f_hanning, min_freq)
                    Pspektrum_hanning_first_peak = np.sqrt(Pspektrum_hanning)[Min_freq_H:Max_freq_H]
                    f_hanning_first_peak = f_hanning[Min_freq_H:Max_freq_H]

                    "In order to find the Full Width Half Max (FWHM) of the first and second peaks"
                    peaks, _ = sps.find_peaks(Pspektrum_hanning_first_peak,distance=len(Pspektrum_hanning_first_peak)/6
                                              ,height=max(Pspektrum_hanning_first_peak)*0.15)
                    FWHM = sps.peak_widths(Pspektrum_hanning_first_peak,peaks,rel_height=0.5)

                    "For interpolation to find FWHM in frequency [Hz]"
                    LowLimit = sorted(FWHM[2])
                    HighLimit = sorted(FWHM[3])

                    LowLimFreq = f_hanning_first_peak[int(LowLimit[-1])]+(LowLimit[-1]-int(LowLimit[-1]))*\
                                 (f_hanning_first_peak[int(LowLimit[-1]+1)]-f_hanning_first_peak[int(LowLimit[-1])])
                    HighLimFreq = f_hanning_first_peak[int(HighLimit[-1])]+(HighLimit[-1]-int(HighLimit[-1]))*\
                                  (f_hanning_first_peak[int(HighLimit[-1]+1)]-f_hanning_first_peak[int(HighLimit[-1])])

                    Errorbars_SecPeak.append([float(f[np.where(Pspektrum == Max_peak)]-LowLimFreq),
                                              float(HighLimFreq-f[np.where(Pspektrum == Max_peak)])])

                    "Finding the errorbars of the first peak frequency"
                    if len(peaks) > 1:
                        First_Peak_frequency.append(f_hanning_first_peak[np.where(Pspektrum_hanning_first_peak
                                                    == sorted(Pspektrum_hanning_first_peak[peaks])[-2])[0]])

                        LowLimFreq = f_hanning_first_peak[int(LowLimit[-2])]+(LowLimit[-2]-int(LowLimit[-2]))*\
                                (f_hanning_first_peak[int(LowLimit[-2]+1)]-f_hanning_first_peak[int(LowLimit[-2])])
                        HighLimFreq = f_hanning_first_peak[int(HighLimit[-2])]+(HighLimit[-2]-int(HighLimit[-2]))*\
                                (f_hanning_first_peak[int(HighLimit[-2]+1)]-f_hanning_first_peak[int(HighLimit[-2])])

                        Errorbars_FirstPeak.append([float(f_hanning_first_peak[np.where(Pspektrum_hanning_first_peak
                                                    == sorted(Pspektrum_hanning_first_peak[peaks])[-2])[0]]-LowLimFreq),
                                            float(HighLimFreq-f_hanning_first_peak[np.where(Pspektrum_hanning_first_peak
                                                    == sorted(Pspektrum_hanning_first_peak[peaks])[-2])[0]])])

                    "Finding the flow velocity"
                elif C_Name == "Flow":
                    Flow = 0.083528531*round(np.mean(channel.data),5)-0.165376967
                    Velocities_Measured.append(Flow/0.15**2)
                    if len(peaks)>1:
                        Velocities_Measured_First_Peak.append(Flow/0.15**2)
            continue
    return MAX_Amplitude, Velocities_Measured, Second_Peak_frequency, First_Peak_frequency,\
           Velocities_Measured_First_Peak,Errorbars_SecPeak, Errorbars_FirstPeak

"The same thing as the above just for the PIV data"
def Welch_func_PIV(Velocities, overlap_None, nperseg_N, Min_plot_freq, Max_plot_freq):
    Pspektrum = []
    f = []
    Pspektrum_max = []
    f_max = []
    Windowtype = []
    Flow_Velocity = []
    Errorbars_PIV = []

    "Finding the positions of the files"

    Files = []
    Velo_Files = []
    for Str in Velocities:
        if Str == '11_1' or Str == '13_1' :
            Files.append("PIV/PD/(40,82)/" + Str + "ms_PD_Pos(40,82)/B00002.dat")
        elif Str == 'Some velocity':
            Files.append("PIV/PD/(10,82)/" + Str + "ms_PD_Pos(10,82)/B00002.dat")
        else:
            Files.append("PIV/PD/(5,82)/" + Str + "ms_PD_Pos(5,82)/B00002.dat")

        Velo_Files.append("PIV/AvgV/" + Str + "ms_AvgV_40mm/B00001.dat")

    "Main of the function"
    for i in range(len(Files)):
        Value = xy_plotting(Files[i])
        Strength = Value[2]

        if nperseg_N == "N":
            nperseg_N_lokke = len(Strength)
        else:
            nperseg_N_lokke = len(Strength)/nperseg_N
            N = len(Strength)

        if Velocities[i] == "Some velocity" :
            f_lokke, Pspektrum_lokke = sps.welch(Strength, 2441, window='hann', nperseg=nperseg_N_lokke,
                                     noverlap=overlap_None, scaling="spectrum")
            f_lokke_N, Pspektrum_lokke_N = sps.welch(Strength, 2441, window='hann', nperseg=len(Strength),
                                     noverlap=overlap_None, scaling="spectrum")
            Windowtype.append('Hanning')
        else:
            f_lokke, Pspektrum_lokke = sps.welch(Strength, 2441, window=('kaiser',2), nperseg=nperseg_N_lokke,
                                     noverlap=overlap_None,scaling="spectrum")
            f_lokke_N, Pspektrum_lokke_N = sps.welch(Strength, 2441, window=('kaiser', 2), nperseg=len(Strength),
                                               noverlap=overlap_None, scaling="spectrum")
            Windowtype.append('Kaiser')

        "Limit the frequency spectrum and add them for plotting"
        #Always have one with the smallest possible window size
        Pspektrum_lokke_N = Pspektrum_lokke_N[maxmin_freq(f_lokke_N, Min_plot_freq):maxmin_freq(f_lokke_N, Max_plot_freq)]
        f_lokke_N = f_lokke_N[maxmin_freq(f_lokke_N, Min_plot_freq):maxmin_freq(f_lokke_N, Max_plot_freq)]

        Pspektrum_lokke = Pspektrum_lokke[maxmin_freq(f_lokke, Min_plot_freq):maxmin_freq(f_lokke, Max_plot_freq)]
        f_lokke = f_lokke[maxmin_freq(f_lokke, Min_plot_freq):maxmin_freq(f_lokke, Max_plot_freq)]
        f.append(f_lokke)
        Pspektrum.append(Pspektrum_lokke)

        "Finding the spectrum peak with corresponding frequency. Only one frequency for the PIV data"
        Pspektrum_max.append(max(Pspektrum_lokke))
        f_max.append(f_lokke[np.where(Pspektrum_lokke==max(Pspektrum_lokke))])

        "Finding the Full Width Half Max from a smaller window type"
        peaks, _ = sps.find_peaks(Pspektrum_lokke,height=max(Pspektrum_lokke))
        FWHM = sps.peak_widths(Pspektrum_lokke,peaks,rel_height=0.5)
        LowLimit = sorted(FWHM[2])
        HighLimit = sorted(FWHM[3])
        LowLimFreq = f_lokke[int(LowLimit[-1])] + (LowLimit[-1] - int(LowLimit[-1])) * \
                     (f_lokke[int(LowLimit[-1] + 1)] - f_lokke[int(LowLimit[-1])])
        HighLimFreq = f_lokke[int(HighLimit[-1])] + (HighLimit[-1] - int(HighLimit[-1])) * \
                      (f_lokke[int(HighLimit[-1] + 1)] - f_lokke[int(HighLimit[-1])])

        "Finding the errorbar limits with the original peak (With N windows) as zeropoint"
        Errorbars_PIV.append([float(f_lokke_N[np.where(Pspektrum_lokke_N == max(Pspektrum_lokke_N))] - LowLimFreq),
                                float(HighLimFreq - f_lokke_N[np.where(Pspektrum_lokke_N == max(Pspektrum_lokke_N))])])

        "Finding the flow velocity from PIV data"
        Velo_value = xy_plotting(Velo_Files[i])
        Flow_Velocity.append(np.mean(Velo_value[1]))

    return Pspektrum, f, Pspektrum_max, f_max, Windowtype, Flow_Velocity, Errorbars_PIV

"""""
Function for finding the position of the max and min frequencies for plotting
"""""
def maxmin_freq(x,maxmin_value):
    for i in range(len(x)):
        if x[i] < maxmin_value:
            continue
        else:
            MAX_MIN = i
            break
    return MAX_MIN

"Function that finds out if string is only made out of numbers or has letters in i too"
def is_float(string):
    """ True if given string is float else False"""
    try:
        return float(string)
    except ValueError:
        return False

"Function that splits the file gotten from PIV post processing so that it is possible to extract data"
def xy_plotting(file_location):

    data = open(file_location, "r")

    Value_1 = []
    Value_2 = []
    Value_3 = []
    Value_4 = []

    for lines in data.readlines():
        "Splits for each space between numbers"
        Value = lines.split(" ")

        if is_float(Value[0]) and is_float(Value[1]) or is_float(Value[0]) and len(Value) > 2 :
            Value_1.append(float(Value[0]))
            Value_2.append(float(Value[1]))
            if len(Value) > 2:
                Value_3.append(float(Value[2]))
                Value_4.append(float(Value[3]))

    return Value_1, Value_2, Value_3, Value_4