"Created by Hasan Besirovic"
"Script which calculates the diffusion rate of the vorticity for the VGs"

import Simple_Functions as SF
import matplotlib.pyplot as plt
import numpy as np
import scipy.signal as sps
from sklearn.linear_model import LinearRegression
from sklearn.preprocessing import PolynomialFeatures
from scipy.optimize import curve_fit

"Choosing the velocity and files"
U_ref = [9.1,10.1,13.1]
x = [45,50,55,60,65,70,75]
Str = ["PIV/Vorticity/9_1ms_Vorticity_SWP_","PIV/Vorticity/10_1ms_Vorticity_SWP_","PIV/Vorticity/13_1ms_Vorticity_SWP_"]

"Getting the files in a specific order"
files = []
MaxAmountPeaks = []
for i in x:
    for n in range(len(Str)):
        files.append(Str[n]+str(i)+"mm/B00001.dat")

"Finding the main peak values for the vorticity plots for the use of checking deday"
for n in range(len(Str)):
    for i in x:
        file = Str[n]+str(i)+"mm/B00001.dat"
        peaks, _ = sps.find_peaks(SF.xy_plotting(file)[1],distance=1280/4)
        valleys,_ = sps.find_peaks([i*-1 for i in SF.xy_plotting(file)[1]],distance=1280/4)
        Extremal = peaks.tolist()
        Extremal.extend(valleys.tolist())
        if len(MaxAmountPeaks) <= n:
            MaxAmountPeaks.append(Extremal)
        elif len(Extremal) > len(MaxAmountPeaks[n]):
            MaxAmountPeaks[n] = Extremal
        files.append(file)

"Appends the value of each peak and valley to a list and normalises it"
k = 0
x_value = []
y_value = []
D = 4.8#mm
for i in range(len(x)):
    for j in range(len(Str)):
        xy = SF.xy_plotting(files[j+k])
        for a in MaxAmountPeaks[j]:
            x_value.append(x[i]/D)
            if xy[1][a]<=0:
                y_value.append(xy[1][a]*-1*D/(1000*U_ref[j]))
            else:
                y_value.append(xy[1][a]*D/(1000*U_ref[j]))

    k = k+len(Str)


"""""
#Just a test to illustrate the peaks and valleys found
plt.plot(xy[0],xy[1])
Test_x = []
Test_y = []
for a in MaxAmountPeaks[0]:
    Test_x.append(xy[0][a])
    Test_y.append(xy[1][a])
plt.plot(Test_x,Test_y,'*')
"""""
def linear(x,a,b):
    return a*(x+b/a)

def exponential(x,a,b):
    return a*np.exp(x)**(b)

fig,_  = plt.subplots(figsize=(8,8.5))

plt.plot(x_value,y_value,'ko',markerfacecolor='None')

popt, pcov = curve_fit(exponential,x_value,y_value)
plt.plot(x_value,exponential(x_value,*popt),label="Exponential. Decay exponent = "+str(round(popt[1],4)))
residuals = y_value-exponential(x_value,*popt)
ss_res = np.sum(residuals**2)
ss_tot = np.sum((y_value-np.mean(y_value))**2)
R = 1-ss_res/ss_tot
print popt, R
popt, pcov = curve_fit(linear,x_value,y_value)
plt.plot(x_value,linear(x_value,*popt),label="Linear. Decay gradient = "+str(round(popt[0],4)))
residuals = y_value-exponential(x_value,*popt)
ss_res = np.sum(residuals**2)
ss_tot = np.sum((y_value-np.mean(y_value))**2)
R = 1-ss_res/ss_tot
print popt, R


"""""
"Linear regression for all the points and also across each velocity"
x_linReg = np.array(x_value).reshape((-1,1))
y_linReg = np.array(y_value)
model = LinearRegression().fit(x_linReg,y_linReg)

x_linReg_1 = np.array(x_value[:len(MaxAmountPeaks[0])*len(x)]).reshape((-1,1))
y_linReg_1 = np.array(y_value[:len(MaxAmountPeaks[0])*len(x)])
model_1 = LinearRegression().fit(x_linReg_1,y_linReg_1)


x_linReg_2 = np.array(x_value[len(MaxAmountPeaks[0])*len(x):len(MaxAmountPeaks[1])*len(x)+
                                                            len(MaxAmountPeaks[0])*len(x)]).reshape((-1,1))
y_linReg_2 = np.array(y_value[len(MaxAmountPeaks[0])*len(x):len(MaxAmountPeaks[1])*len(x)+
                                                            len(MaxAmountPeaks[0])*len(x)])
model_2 = LinearRegression().fit(x_linReg_2,y_linReg_2)


x_linReg_3 = np.array(x_value[len(MaxAmountPeaks[1])*len(x)+len(MaxAmountPeaks[0])*len(x):
            len(MaxAmountPeaks[1])*len(x)+len(MaxAmountPeaks[0])*len(x)+len(MaxAmountPeaks[2])*len(x)]).reshape((-1,1))
y_linReg_3 = np.array(y_value[len(MaxAmountPeaks[1])*len(x)+len(MaxAmountPeaks[0])*len(x):
            len(MaxAmountPeaks[1])*len(x)+len(MaxAmountPeaks[0])*len(x)+len(MaxAmountPeaks[2])*len(x)])
model_3 = LinearRegression().fit(x_linReg_3,y_linReg_3)

"plotting both the points and corresponding linear regression lines"
fig,_  = plt.subplots(figsize=(8,8.5))

plt.plot(x_value,model.predict(x_linReg),'r-',label="Total Average, Decay gradient = "+str(round(model.coef_,4)))

plt.plot(x_value[:len(MaxAmountPeaks[0])*len(x)],model_1.predict(x_linReg_1),'k--',marker='',
         label="Average 9.1m/s, Decay gradient = "+str(round(model_1.coef_,4)))
plt.plot(x_value[:len(MaxAmountPeaks[0])*len(x)],y_value[:len(MaxAmountPeaks[0])*len(x)],'k*')

plt.plot(x_value[len(MaxAmountPeaks[0])*len(x):len(MaxAmountPeaks[1])*len(x)+len(MaxAmountPeaks[0])*len(x)],
         model_2.predict(x_linReg_2),'k:',marker='o',markerfacecolor ='None',
        label="Average 10.1m/s, Decay gradient = "+str(round(model_2.coef_,4)))
plt.plot(x_value[len(MaxAmountPeaks[0])*len(x):len(MaxAmountPeaks[1])*len(x)+len(MaxAmountPeaks[0])*len(x)],
         y_value[len(MaxAmountPeaks[0])*len(x):len(MaxAmountPeaks[1])*len(x)+len(MaxAmountPeaks[0])*len(x)],
         'k',marker='o',linestyle='None',markerfacecolor ='None')

plt.plot(x_value[len(MaxAmountPeaks[1])*len(x)+len(MaxAmountPeaks[0])*len(x):len(MaxAmountPeaks[1])*len(x)+
                len(MaxAmountPeaks[0])*len(x)+len(MaxAmountPeaks[2])*len(x)],model_3.predict(x_linReg_3),
         'k.-.',marker='s',markerfacecolor ='None',label="Average 13.1m/s, Decay gradient = "+str(round(model_3.coef_,4)))
plt.plot(x_value[len(MaxAmountPeaks[1])*len(x)+len(MaxAmountPeaks[0])*len(x):len(MaxAmountPeaks[1])*
                                                len(x)+len(MaxAmountPeaks[0])*len(x)+len(MaxAmountPeaks[2])*len(x)],
         y_value[len(MaxAmountPeaks[1])*len(x)+len(MaxAmountPeaks[0])*len(x):len(MaxAmountPeaks[1])*
                                                len(x)+len(MaxAmountPeaks[0])*len(x)+len(MaxAmountPeaks[2])*len(x)],
         'k',marker='s',linestyle='None',markerfacecolor ='None')
"""""
plt.ylabel(r'$\dfrac{\mid Vorticity \mid \cdot D}{U_{ref}}$',fontsize=14)
plt.xlabel('x/D',fontsize=14)
plt.legend(fontsize = 16)
plt.savefig("PICS/GraphsToUse/VortexDecay_SWP.pdf")
plt.show()


