"Created by Hasan Besirovic"
"Script which calculates the diffusion rate of the vorticity for the von karman street"

import Simple_Functions as SF
import matplotlib.pyplot as plt
import numpy as np
import scipy.signal as sps
from sklearn.linear_model import LinearRegression
from sklearn.preprocessing import PolynomialFeatures
from scipy.optimize import curve_fit

"Specifying which files to get"
U_ref = [8,9.1,9.6,10.1,10.6,11.1,13.1]
x = [30,35,40,45,50,55,60,65,70]
Str = ["PIV/Vorticity/8ms_Vorticity_","PIV/Vorticity/9_1ms_Vorticity_","PIV/Vorticity/9_6ms_Vorticity_",
       "PIV/Vorticity/10_1ms_Vorticity_","PIV/Vorticity/10_6ms_Vorticity_","PIV/Vorticity/11_1ms_Vorticity_",
       "PIV/Vorticity/13_1ms_Vorticity_"]

"Getting the files in a specific order"
files = []
MaxAmountPeaks = []
for i in x:
    for n in range(len(Str)):
        files.append(Str[n]+str(i)+"mm/B00001.dat")

"Finding the peak and valley for the data"
for n in range(len(Str)):
    for i in x:
        file = Str[n]+str(i)+"mm/B00001.dat"
        peaks, _ = sps.find_peaks(SF.xy_plotting(file)[1],distance=1280)
        valleys,_ = sps.find_peaks([i*-1 for i in SF.xy_plotting(file)[1]],distance=1280)
        Extremal = peaks.tolist()
        Extremal.extend(valleys.tolist())
        if len(MaxAmountPeaks) <= n:
            MaxAmountPeaks.append(Extremal)
        elif len(Extremal) > len(MaxAmountPeaks[n]):
            MaxAmountPeaks[n] = Extremal

"Appending the values which are needed for plotting and linear regression and which are normalised "
k = 0
x_value = []
y_value = []
D = 4.8
for i in range(len(x)):
    for j in range(len(Str)):
        xy = SF.xy_plotting(files[j+k])
        for a in MaxAmountPeaks[j]:
            x_value.append(x[i]/D)
            if xy[1][a]<=0:
                y_value.append(xy[1][a]*-1*D/(1000*U_ref[j]))
            else:
                y_value.append(xy[1][a]*D/(1000*U_ref[j]))
    k = k+len(Str)

"""""
Just a test to illustrate the peaks and valleys found

plt.plot(xy[0],xy[1])
Test_x = []
Test_y = []
for a in MaxAmountPeaks[0]:
    Test_x.append(xy[0][a])
    Test_y.append(xy[1][a])
plt.plot(Test_x,Test_y,'*')
"""""

"For the curvefit of a specific function"
def hypFunc(x,a,b):
    return a/(x+b)

def exponential(x,a,b):
    return a*np.exp(x)**(b)

fig,_  = plt.subplots(figsize=(8,6))

popt, pcov = curve_fit(hypFunc,x_value,y_value)
#plt.plot(x_value,hypFunc(x_value,*popt),label="Hyperbolic")
residuals = y_value-hypFunc(x_value,*popt)
ss_res = np.sum(residuals**2)
ss_tot = np.sum((y_value-np.mean(y_value))**2)
R = 1-ss_res/ss_tot
print popt, R
popt, pcov = curve_fit(exponential,x_value,y_value)
plt.plot(x_value,exponential(x_value,*popt),label="Exponential")
residuals = y_value-exponential(x_value,*popt)
ss_res = np.sum(residuals**2)
ss_tot = np.sum((y_value-np.mean(y_value))**2)
R = 1-ss_res/ss_tot
print popt, R

"For the polynomial regression"
x_linReg = np.array(x_value).reshape((-1,1))
y_linReg = np.array(y_value)

Transform = PolynomialFeatures(degree=2,include_bias=False).fit_transform(x_linReg)
model = LinearRegression().fit(Transform,y_linReg)
intercept, coefficients = model.intercept_, model.coef_
#print intercept, coefficients

#plt.plot(x_value,model.predict(Transform),'r-',label="Total Average, dw/dx = ")
plt.plot(x_value,y_value,'ks', markerfacecolor ='None')


plt.ylabel(r'$\dfrac{\mid Vorticity \mid \cdot D}{U_{ref}}$', fontsize=14)
plt.xlabel('x/D',fontsize=14)
plt.legend(fontsize=16)

plt.savefig("PICS/GraphsToUse/VortexDecay_SYM.pdf")
plt.show()