"Created by Hasan Besirovic"
"Script which calculates the WelchPowerSpectrum for a set of strain gauge data"

import Simple_Functions as SF
import matplotlib.pyplot as plt
import numpy as np
import matplotlib.lines as lin
import scipy.signal as sps

"Gets data that is wanted"
Datagroups = ["6ms","8ms","9_1ms","11_1ms"]
Tdms_file = ["STRAIN_GAUGE/VG_Foil/SymPlaneMeasurements/RampDown.tdms",
             "STRAIN_GAUGE/VG_Foil/SymPlaneMeasurements/RampDown.tdms"]
"Limits the data to these frequencies"
Max_plot_freq = 1000
Min_plot_freq = 200

"Welch with a chosen window with 50% overlap"
data = SF.Welch_func(Tdms_file,Datagroups,None,"N",Min_plot_freq,Max_plot_freq,0)
Pspektrum = data[0]
f = data[1]
Pspektrum_max = data[2]
f_max = data[3]
Velocity = data[6]
Windowtype = data[7]
"Welch with a bigger window"
SamplesPerWindow = 1000
data = SF.Welch_func(Tdms_file,Datagroups,None,SamplesPerWindow,Min_plot_freq,Max_plot_freq,0)
Pspektrum_N = data[0]
f_N = data[1]
Pspektrum_max_N = data[2]
f_max_N = data[3]

"Plotting"
Numbering = ["a)","b)","c)","d)","e)","f)","g)","h)","i)","k)","l)","m)","n)","o)","p)"]
k = 0
fig, axs = plt.subplots(len(Tdms_file),len(Datagroups) / len(Tdms_file),sharex = True, sharey = True,figsize=(14.4,6))
for i in range(len(Tdms_file)):
    for j in range(len(Datagroups)/len(Tdms_file)):


        "For one spectrum (1-Figure)"
        if len(Datagroups) == 1:
            peaks, _ = sps.find_peaks(Pspektrum_N[j+k], distance=len(Pspektrum_N[j+k])/6,
                                      height=Pspektrum_max_N[j+k]*0.1)
            "First Peak from the one with bigger hanning window"
            if len(peaks)>1:
                FirstPeak = f_N[j+k][np.where(Pspektrum_N[j+k]==sorted(Pspektrum_N[j+k][peaks])[-2])[0]]
                plt.annotate("(" + str(round(FirstPeak,2)) + "Hz)",
                             xy=(FirstPeak,sorted(Pspektrum_N[j+k][peaks])[-2]/Pspektrum_max_N[j+k]*1.15),fontsize=12)

            plt.plot(f[j+k],np.abs(Pspektrum[j+k])/Pspektrum_max[j+k], color="darkgray")
            plt.plot(f_N[j+k],np.abs(Pspektrum_N[j+k])/Pspektrum_max_N[j+k], color="k")
            plt.annotate("(" + str(round(f_max[j + k], 2)) + "Hz)",
                         xy=(f_max[j + k]+10, Pspektrum_max[j + k]/(2*Pspektrum_max[j+k])),fontsize=12)
            #plt.title(Numbering[j+k])
            plt.title(str(round(Velocity[j+k],2)) + r" $\frac{m}{s}$ ")
            lines = [lin.Line2D([0], [0], color="k"), lin.Line2D([0], [0], color="darkgray")]
            plt.legend((lines), (Windowtype[j+k]+": n = 1", Windowtype[j+k]+
                                 ": n = " + str(SamplesPerWindow)),loc = "upper left",fontsize=12)


            "For one file with several velocities"
        elif len(Tdms_file) < 2:
            peaks, _ = sps.find_peaks(Pspektrum_N[j+k], distance=len(Pspektrum_N[j+k])/6,
                                      height=Pspektrum_max_N[j+k]*0.1)
            "First Peak from the one with bigger hanning window"
            if len(peaks)>1:
                FirstPeak = f_N[j+k][np.where(Pspektrum_N[j+k]==sorted(Pspektrum_N[j+k][peaks])[-2])[0]]
                axs[j].annotate("("+ str(round(FirstPeak,2)) + "Hz)",
                            xy=(FirstPeak,sorted(Pspektrum_N[j+k][peaks])[-2]/Pspektrum_max_N[j+k]*1.15),fontsize=12)

            axs[j].plot(f[j+k],np.abs(Pspektrum[j+k])/Pspektrum_max[j+k], color="darkgray")
            axs[j].plot(f_N[j+k],np.abs(Pspektrum_N[j+k])/Pspektrum_max_N[j+k], color="k")
            axs[j].annotate("(" + str(round(f_max[j + k], 2)) + "Hz)",
                            xy=(f_max[j + k]*1.07,Pspektrum_max[j + k]/(2*Pspektrum_max[j+k])),fontsize=12)
            #axs[j].set_title(Numbering[j+k])
            axs[j].set_title(str(round(Velocity[j+k],2)) + r" $\frac{m}{s}$ ")
            lines = [lin.Line2D([0], [0], color="k"), lin.Line2D([0], [0], color="darkgray")]
            axs[j].legend((lines), (Windowtype[j+k]+": n = 1", Windowtype[j+k]+
                                    ": n = " + str(SamplesPerWindow)),loc = "upper left",fontsize=12)


            "For several files with several velocities"
        else:
            peaks, _ = sps.find_peaks(Pspektrum_N[j+k], distance= len(Pspektrum_N[j+k])/6,
                                      height=Pspektrum_max_N[j+k]*0.1)
            "First Peak from the one with bigger hanning window"
            if len(peaks)>1:
                FirstPeak = f_N[j+k][np.where(Pspektrum_N[j+k]==sorted(Pspektrum_N[j+k][peaks])[-2])[0]]
                axs[i,j].annotate("(" + str(round(FirstPeak,2)) + "Hz)",
                            xy=(FirstPeak*0.98,sorted(Pspektrum_N[j+k][peaks])[-2]/Pspektrum_max_N[j+k]*1.15),fontsize=13)
                print sorted(Pspektrum_N[j+k][peaks])[-2]

            axs[i, j].plot(f[j + k], np.abs(Pspektrum[j + k])/Pspektrum_max[j + k], color="darkgray")
            axs[i, j].plot(f_N[j + k], np.abs(Pspektrum_N[j + k])/Pspektrum_max_N[j + k], color="k")
            axs[i,j].text(800,0.9, "A$_{max}$ = " + str(round(Pspektrum_max_N[j + k],7)),fontsize=13,
                     bbox=dict(facecolor='none', edgecolor='lightgray'))
            axs[i,j].annotate("(" + str(round(f_max[j + k], 2)) + "Hz)",
                              xy=(f_max[j + k]*1.07,Pspektrum_max[j + k]/(2*Pspektrum_max[j+k])),fontsize=13)
            axs[i,j].set_title(Numbering[j+k],fontsize=18)
            #axs[i,j].set_title(str(round(Velocity[j+k],2)) + r" $\frac{m}{s}$ ")
            lines = [lin.Line2D([0], [0], color="k"), lin.Line2D([0], [0], color="darkgray")]
            axs[i,j].legend((lines), (Windowtype[j+k]+": n = 1", Windowtype[j+k]+
                                      ": n = " + str(SamplesPerWindow)),loc = "upper left",fontsize=12)
            axs[i,j].tick_params(labelsize=12)


    k = k + len(Datagroups) / len(Tdms_file)

print Velocity

"To add common y and x labels"
fig.add_subplot(111, frameon=False)
plt.tick_params(labelcolor='none', top=False, bottom=False, left=False, right=False)
plt.xlabel("Frequency [Hz]",fontsize=14)
plt.ylabel("A/$A_{max}$ [RMS]",fontsize=14)

fig.tight_layout()

plt.savefig("PICS/GraphsToUse/Welch_PowerSpektrum_SymPlane.pdf")
plt.show()

