"Created by Hasan Besirovic"
"Script which calculates the Welch Power spectrum for a set of PIV data"

import matplotlib.pyplot as plt
import Simple_Functions as SF
import matplotlib.lines as lin

"Choosing velocities and limiting the frequencies for which data is plotted"
Velocities = ["8", "9_1", "9_6", "10_1", "10_6", "11_1"]
Min_plot_freq = 150
Max_plot_freq = 1000

"Full Welch method with smallest window possible, one window for each sample"
data = SF.Welch_func_PIV(Velocities,None,"N",Min_plot_freq,Max_plot_freq)
Pspektrum = data[0]
f = data[1]
Pspektrum_max = data[2]
f_max = data[3]
Windowtype = data[4]
"Welch's method with a larger window, one window for each SamplesPerWindow"
SamplesPerWindow = 100
data = SF.Welch_func_PIV(Velocities,None,SamplesPerWindow,Min_plot_freq,Max_plot_freq)
Pspektrum_N = data[0]
f_N = data[1]
Pspektrum_max_N = data[2]
f_max_N = data[3]

"Plotting"
Numbering = ["a)","b)","c)","d)","e)","f)","g)","h)","i)","k)","l)","m)","n)","o)","p)"]
fig,axs = plt.subplots(len(Velocities)/2,2,sharex = True, sharey = True, figsize = (14.4,8))
k=0
for i in range(len(Velocities)/2):
        for j in range(2):
                axs[i,j].plot(f[j+k],Pspektrum[j+k]/Pspektrum_max[j+k], color = "darkgray")
                axs[i,j].plot(f_N[j+k],Pspektrum_N[j+k]/Pspektrum_max_N[j+k],color = "k")
                axs[i,j].set_title(Numbering[j+k],fontsize=18)
                #axs[i,j].set_title(Velocities[j+k]+" m/s")
                axs[i,j].annotate("("+str(round(f_max[j + k], 2)) + "Hz)",
                                  xy=(f_max[j + k]*1.18,Pspektrum_max[j + k]/(Pspektrum_max[j + k])*0.9),fontsize=13)
                axs[i,j].text(130, 0.57, "A$_{max}$ = " + str(round(Pspektrum_max[j + k], 5)),
                         bbox=dict(facecolor='none', edgecolor='lightgray'))
                "Creating the legend"
                lines = [lin.Line2D([0], [0], color="darkgray"), lin.Line2D([0], [0], color="k")]
                axs[i,j].legend((lines),(Windowtype[j+k]+": n = 1", Windowtype[j+k]+": n = " + str(SamplesPerWindow)),
                                loc = "upper left",fontsize=12)
        k = k+2

"To add common y and x labels"
fig.add_subplot(111, frameon=False)
plt.tick_params(labelcolor='none', top=False, bottom=False, left=False, right=False)
plt.xlabel("Frequency [Hz]",fontsize=14)
plt.ylabel("A/$A_{max}$",fontsize=14)

fig.tight_layout()
plt.savefig("PICS/GraphsToUse/Welch_PowerSpektrum_SymPlane_PIV.pdf")
plt.show()