"Written by Hasan Besriovic - Spring 2020"
"Script which extracts data from PIV data plots and plots it"

import matplotlib.pyplot as plt
import Simple_Functions as SF
import os

"Specify which data to extract and calculate the average velocity from velocity plot in symmetry plane"
Str_1 = "PIV/AvgV/"
Str_2 = ["ms_AvgV_SWP_70mm/B00002.dat"]#,"ms_AvgV_SWP_70mm/B00002.dat"]
Str_3 = ["9_1","10_1","13_1"]
Standardeviation = ["ms_AvgV_SWP_70mm/B00002.dat"]#,"ms_AvgV_SWP_70mm/B00002.dat"]
AbsVelo = ["ms_AvgV_SWP_70mm/B00003.dat"]#,"ms_AvgV_SWP_70mm/B00003.dat"]
Uncertainty_loc = "PIV/Uncertainty/xy-plot/SWP/"
Uncertainty_Str =  ["ms_Uncertainty_Vy_70mm/B00001.dat"]

Files = []
ErrorFiles = []
AvgVeloFiles = []
for i in range(len(Str_2)):
    for Str in Str_3:
        Files.append(Str_1+Str+Str_2[i])
        ErrorFiles.append((Uncertainty_loc+Str+Uncertainty_Str[i]))
        AvgVeloFiles.append(Str_1+Str+AbsVelo[i])

"Plotting"
fig,ax = plt.subplots(1,len(Str_2), figsize = (10,6))

"For the legend"
k=0
Ref_Velo = [9.1,10.1,13.1]
for i in range(len(Ref_Velo)):
    Ref_Velo[i] = "$U_{ref}$ = "+str(Ref_Velo[i])+"m/s"

D = 4.8
for i in range(len(Str_2)):
    for j in range(len(Str_3)):
        xy = SF.xy_plotting(Files[j+k])
        Avg = SF.xy_plotting(AvgVeloFiles[j+k])
        y = [Value/D for Value in xy[0]]
        x = [(Value-(sum(Avg[1])/len(Avg[1])))/(sum(Avg[1])/len(Avg[1])) for Value in xy[1]]

        "Calculating the error and add for errorbars"
        if os.path.isfile(ErrorFiles[j+k]):
            Velo_error = [(Value)/(12*D*sum(Avg[1])/len(Avg[1]))
                          for Value in SF.xy_plotting(ErrorFiles[j+k])[1]]
            while len(Velo_error)<len(x):
                Velo_error.append(Velo_error[-1])

        "Choose which data has errorbars"
        if len(Str_2)<2:
            if Str_3[j] == "10_1":
                plt.errorbar(x,y,xerr=Velo_error,capsize=2,label = Ref_Velo[j],marker='o',markerfacecolor ='None')
            else:
                plt.plot(x,y,label=Ref_Velo[j])
            plt.xlim(-1.020, -0.99)
            plt.grid()
            "For several plots in the same figure"
        else:
            if Str_3[j] == "10_1":
                ax[i].errorbar(x,y,xerr=Velo_error,capsize=2,label = Ref_Velo[j],marker='o',markerfacecolor ='None')
            else:
                ax[i].plot(x,y,label=Ref_Velo[j])
            ax[i].set_xlim(-1.020, -0.99)
            ax[i].grid()
            ax[i].set_title(Str_2[i].split('ms_AvgV_')[1].split('/')[0])

    k = k + len(Str_3)

"To organise the legend such that it is alphabethical"
handles, labels = ax.get_legend_handles_labels()
handles_sort = []
for i in range(len(labels)):
    handles_sort.append(handles[labels.index(Ref_Velo[i])])

plt.legend(handles_sort,Ref_Velo,fontsize = 14,loc='center right')

"To add common y and x labels"
fig.add_subplot(111, frameon=False)
plt.tick_params(labelcolor='none', top=False, bottom=False, left=False, right=False)
plt.ylabel("z/D",fontsize = 20)
plt.xlabel(r"$\dfrac{U_z-U_{ref}}{U_{ref}}$", fontsize=20)

fig.tight_layout()
plt.savefig("PICS/GraphsToUse/AverageVelocity_Vy_SWP_70mm.pdf")
plt.show()
